/*! Built with http://stenciljs.com */
import { h } from './bscomponents.core.js';
import { p as _size, s as addClass, t as hasClass, r as removeClass, q as customEvent } from './chunk-89dac778.js';
import { c as _has } from './chunk-93432030.js';
import { c as toNumber, d as isNaN$1 } from './chunk-64cf3b78.js';
import { a as _isElement } from './chunk-55988a92.js';
import { a as getUniqueId } from './chunk-ddbf79d9.js';
import { a as closest, b as elementMatches } from './chunk-5028a50e.js';
import { a as getTargetSelector } from './chunk-e1923c80.js';
import './chunk-5260d582.js';
var BsScrollspy = /** @class */ (function () {
    function BsScrollspy() {
        var _this = this;
        this.activateEventName = 'activate.bs.scrollspy';
        this.useBodyForScrollElement = false;
        this.verboseLogging = false;
        this.defaults = {
            offset: 10,
            method: 'auto',
            target: '',
        };
        this.handleScrollElement = function () { return _this.process(); };
    }
    BsScrollspy.prototype.componentWillLoad = function () {
        this.scrollElement = this.useBodyForScrollElement === true ? window : this.scrollspyEl;
        this.config = this.getConfig();
        this.selector = this.config.target + " .nav-link," + this.config.target + " .list-group-item," + this.config.target + " .dropdown-item";
        this.offsets = [];
        this.targets = [];
        this.activeTarget = null;
        this.scrollHeight = 0;
        this.scrollElement.addEventListener('scroll', this.handleScrollElement);
        this.refresh();
        this.process();
    };
    BsScrollspy.prototype.componentDidUnload = function () {
        if (this.scrollElement) {
            this.scrollElement.removeEventListener('scroll', this.handleScrollElement);
        }
        this.scrollElement = null;
        this.config = null;
        this.selector = null;
        this.offsets = null;
        this.targets = null;
        this.activeTarget = null;
        this.scrollHeight = null;
    };
    BsScrollspy.prototype.getConfig = function (overrideConfig) {
        if (overrideConfig === void 0) { overrideConfig = {}; }
        var config = {};
        if (_has(overrideConfig, 'target')) {
            config.target = overrideConfig.target;
        }
        else if (_has(this.scrollspyEl.dataset, 'target')) {
            config.target = this.scrollspyEl.dataset.target;
        }
        else {
            config.target = this.defaults.target;
        }
        if (_isElement(config.target)) {
            var id = config.target.getAttribute('id');
            if (!id) {
                id = getUniqueId('scrollspy');
                config.target.setAttribute('id', id);
            }
            config.target = "#" + id;
        }
        if (_has(overrideConfig, 'offset')) {
            config.offset = toNumber(overrideConfig.offset);
        }
        else if (_has(this.scrollspyEl.dataset, 'offset')) {
            config.offset = toNumber(this.scrollspyEl.dataset.offset);
        }
        else {
            config.offset = this.defaults.offset;
        }
        if (isNaN$1(config.offset)) {
            config.offset = this.defaults.offset;
        }
        if (_has(overrideConfig, 'method')) {
            config.method = toNumber(overrideConfig.method);
        }
        else if (_has(this.scrollspyEl.dataset, 'method')) {
            config.method = this.scrollspyEl.dataset.method;
        }
        else {
            config.method = this.defaults.method;
        }
        return config;
    };
    /**
     * Get an element's distance from the top of the Document.
     * @private
     * @param  {Node} elem The element
     * @return {Number}    Distance from the top in pixels
     */
    BsScrollspy.prototype.getOffsetTop = function (element) {
        // original: https://github.com/cferdinandi/gumshoe/blob/master/src/js/gumshoe.js#L138
        var location = 0;
        var currentEl = element;
        if (currentEl.offsetParent) {
            do {
                location += currentEl.offsetTop;
                currentEl = currentEl.offsetParent;
            } while (currentEl);
        }
        else {
            location = currentEl.offsetTop;
        }
        // location = location - headerHeight - this.offset;
        location -= this.config.offset;
        return location >= 0 ? location : 0;
    };
    BsScrollspy.prototype.refresh = function () {
        var _this = this;
        if (!this.config.target || _size(this.config.target) === 0) {
            return;
        }
        // const autoMethod = this.scrollElement === this.scrollElement.window ? 'offset' : 'position';
        // const offsetMethod = this.config.method === 'auto' ? autoMethod : this.config.method;
        // const offsetBase = offsetMethod === 'position' ? this.getScrollTop() : 0;
        this.offsets = [];
        this.targets = [];
        this.scrollHeight = this.getScrollHeight();
        // const scrollspyBCR = this.scrollElement.getBoundingClientRect();
        var scrollElementTop = this.getOffsetTop(this.scrollElement);
        var container = document.querySelector(this.config.target);
        if (!container) {
            console.warn("unable to locate target selector \"" + this.config.target + "\"");
            return;
        }
        var targets = Array.prototype.slice.call(container.querySelectorAll(this.selector));
        targets.map(function (element) {
            var target;
            var targetSelector = getTargetSelector(element);
            if (targetSelector) {
                target = document.querySelector(targetSelector);
            }
            if (target) {
                var targetBCR = target.getBoundingClientRect();
                if (targetBCR.width || targetBCR.height) {
                    var targetTop = _this.getOffsetTop(target);
                    // const returnVal = [targetBCR.top - scrollspyBCR.top, targetSelector];
                    var returnVal = [targetTop - scrollElementTop, targetSelector];
                    // if (this.verboseLogging) {
                    //   console.log('returnVal: ', returnVal);
                    //   // console.log('this.getOffsetTop(this.scrollElement): ', this.getOffsetTop(this.scrollElement));
                    //   // console.log('this.getOffsetTop(target): ', this.getOffsetTop(target));
                    // }
                    return returnVal;
                }
            }
            return null;
        })
            .filter(function (item) { return item; })
            .sort(function (a, b) { return a[0] - b[0]; })
            .forEach(function (item) {
            _this.offsets.push(item[0]);
            _this.targets.push(item[1]);
        });
    };
    BsScrollspy.prototype.getScrollTop = function () {
        if (this.scrollElement === window) {
            return this.scrollElement.pageYOffset;
        }
        return this.scrollElement.scrollTop;
    };
    BsScrollspy.prototype.getScrollHeight = function () {
        return this.scrollElement.scrollHeight || Math.max(document.body.scrollHeight, document.documentElement.scrollHeight);
    };
    BsScrollspy.prototype.getOffsetHeight = function () {
        if (this.scrollElement === window) {
            return window.innerHeight;
        }
        return this.scrollElement.getBoundingClientRect().height;
    };
    BsScrollspy.prototype.isActiveTarget = function (index, scrollTop) {
        if (this.activeTarget === this.targets[index]) {
            return false;
        }
        if (scrollTop < this.offsets[index]) {
            return false;
        }
        if (typeof this.offsets[index + 1] === 'undefined') {
            if (this.verboseLogging) {
                console.log("at end of list activating \"" + this.targets[index] + "\"");
            }
            return true;
        }
        if (scrollTop < this.offsets[index + 1]) {
            if (this.verboseLogging) {
                console.log("scrollTop of " + scrollTop + " in range of " + this.offsets[index] + " - " + this.offsets[index + 1] + " activating \"" + this.targets[index] + "\"");
            }
            return true;
        }
        return false;
    };
    BsScrollspy.prototype.process = function () {
        if (!this.config.target || _size(this.config.target) === 0) {
            return;
        }
        var scrollTop = this.getScrollTop() + this.config.offset;
        if (this.verboseLogging) {
            console.log('scrollTop + offset = ', scrollTop);
        }
        var scrollHeight = this.getScrollHeight();
        var maxScroll = this.config.offset + scrollHeight - this.getOffsetHeight();
        var container = document.querySelector(this.config.target);
        if (!container) {
            return;
        }
        if (this.scrollHeight !== scrollHeight) {
            this.refresh();
        }
        if (scrollTop >= maxScroll) {
            var target = this.targets[this.targets.length - 1];
            if (this.activeTarget !== target) {
                this.activate(target);
            }
            return;
        }
        if (this.activeTarget && scrollTop < this.offsets[0] && this.offsets[0] > 0) {
            this.activeTarget = null;
            this.clear(container);
            return;
        }
        console.log('this.offsets: ', this.offsets);
        console.log('this.targets: ', this.targets);
        for (var i = _size(this.offsets) - 1; i >= 0; i -= 1) {
            if (this.isActiveTarget(i, scrollTop)) {
                this.activate(this.targets[i]);
            }
        }
    };
    BsScrollspy.prototype.clear = function (container) {
        var nodeArr = Array.prototype.slice.call(container.querySelectorAll(this.selector));
        for (var j = 0; j < nodeArr.length; j += 1) {
            if (hasClass(nodeArr[j], 'active')) {
                removeClass(nodeArr[j], 'active');
            }
        }
    };
    BsScrollspy.getLinkArr = function (target, container) {
        var activeTargetEl = document.querySelector(target);
        if (!activeTargetEl) {
            return [];
        }
        var matchingLinks = [];
        var possibleLinkArr = Array.prototype.slice.call(container.querySelectorAll('.nav-link, .list-group-item, .dropdown-item'));
        for (var j = 0; j < possibleLinkArr.length; j += 1) {
            var targetSelector = getTargetSelector(possibleLinkArr[j]);
            if (targetSelector && elementMatches(activeTargetEl, targetSelector)) {
                matchingLinks.push(possibleLinkArr[j]);
            }
        }
        return matchingLinks;
    };
    BsScrollspy.getParentArrBySelector = function (el, selector) {
        var returnArr = [];
        var closestMatching;
        var currentEl = el.parentElement || el.parentNode;
        do {
            closestMatching = closest(currentEl, selector);
            if (closestMatching) {
                returnArr.push(closestMatching);
                currentEl = closestMatching.parentElement || closestMatching.parentNode;
            }
        } while (closestMatching);
        return returnArr;
    };
    BsScrollspy.prototype.activate = function (target) {
        if (!this.config.target || _size(this.config.target) === 0) {
            return;
        }
        var relatedTarget;
        this.activeTarget = target;
        var container = document.querySelector(this.config.target);
        this.clear(container);
        var linkArr = BsScrollspy.getLinkArr(target, container);
        for (var j = 0; j < linkArr.length; j += 1) {
            if (hasClass(linkArr[j], 'dropdown-item')) {
                var closestDropdown = closest(linkArr[j], '.dropdown');
                if (closestDropdown && container.contains(closestDropdown)) {
                    var dropdownToggle = closestDropdown.querySelector('.dropdown-toggle');
                    if (dropdownToggle) {
                        addClass(dropdownToggle, 'active');
                    }
                }
                addClass(linkArr[j], 'active');
                relatedTarget = linkArr[j];
            }
            else {
                addClass(linkArr[j], 'active');
                relatedTarget = linkArr[j];
                var parentSelector = '.nav, .list-group';
                var currentEl = linkArr[j];
                do {
                    currentEl = currentEl.parentElement || currentEl.parentNode;
                    currentEl = closest(currentEl, parentSelector);
                    var prevEl = currentEl ? currentEl.previousElementSibling : null;
                    if (currentEl && prevEl) {
                        if (elementMatches(prevEl, '.nav-link, .list-group-item')) {
                            addClass(currentEl.previousElementSibling, 'active');
                        }
                        // Handle special case when .nav-link is inside .nav-item
                        if (elementMatches(prevEl, '.nav-item')) {
                            var listItemChildren = Array.prototype.slice.call(prevEl.children);
                            for (var x = 0; x < listItemChildren.length; x += 1) {
                                if (elementMatches(listItemChildren[x], '.nav-link')) {
                                    addClass(listItemChildren[x], 'active');
                                }
                            }
                        }
                    }
                } while (currentEl && container.contains(currentEl));
            }
        }
        customEvent(this.scrollElement, this.activateEventName, { target: target }, relatedTarget);
    };
    BsScrollspy.prototype.scrollspy = function (scrollspyOptions) {
        if (scrollspyOptions === void 0) { scrollspyOptions = {}; }
        if (_size(scrollspyOptions) === 0) {
            return this.scrollspyEl;
        }
        if (scrollspyOptions === 'refresh') {
            this.refresh();
            return true;
        }
        if (scrollspyOptions === 'getActiveTarget') {
            return this.activeTarget;
        }
        if (typeof scrollspyOptions === 'object') {
            this.config = this.getConfig(scrollspyOptions);
            this.refresh();
            this.process();
            return true;
        }
        if (typeof scrollspyOptions === 'string') {
            throw new Error("No method named \"" + scrollspyOptions + "\"");
        }
        return null;
    };
    BsScrollspy.prototype.render = function () {
        return (h("slot", null));
    };
    Object.defineProperty(BsScrollspy, "is", {
        get: function () { return "bs-scrollspy"; },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BsScrollspy, "properties", {
        get: function () {
            return {
                "activateEventName": {
                    "type": String,
                    "attr": "activate-event-name"
                },
                "activeTarget": {
                    "state": true
                },
                "config": {
                    "state": true
                },
                "defaults": {
                    "type": "Any",
                    "attr": "defaults"
                },
                "offsets": {
                    "state": true
                },
                "scrollElement": {
                    "state": true
                },
                "scrollHeight": {
                    "state": true
                },
                "scrollspy": {
                    "method": true
                },
                "scrollspyEl": {
                    "elementRef": true
                },
                "selector": {
                    "state": true
                },
                "targets": {
                    "state": true
                },
                "useBodyForScrollElement": {
                    "type": Boolean,
                    "attr": "use-body-for-scroll-element"
                },
                "verboseLogging": {
                    "type": Boolean,
                    "attr": "verbose-logging"
                }
            };
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BsScrollspy, "style", {
        get: function () { return "bs-scrollspy {\n  display: block;\n}"; },
        enumerable: true,
        configurable: true
    });
    return BsScrollspy;
}());
export { BsScrollspy };
